import net.mikekohn.java_grinder.CPU;
import net.mikekohn.java_grinder.IOPort0;
import net.mikekohn.java_grinder.IOPort1;
import net.mikekohn.java_grinder.IOPort2;
import net.mikekohn.java_grinder.IOPort3;
import net.mikekohn.java_grinder.IOPort4;
import net.mikekohn.java_grinder.IOPort5;
import net.mikekohn.java_grinder.IOPort6;
import net.mikekohn.java_grinder.Timer;
import net.mikekohn.java_grinder.TimerListener;

public class VGA implements TimerListener
{
  static final int HSYNC = 1;
  static final int VSYNC = 2;
  static int sync = 4|2|1;
  static int port0_value = 0x01;
  static int port3_value = 0x00;
  static int pos_y = 0;
  static byte[] image;
  static int ptr = 0;
  static int state = 0;
  static int frame_count = 0;

  static public void main(String args[])
  {
    // 25MHz clock
    CPU.setClockExternal2();
    //CPU.setClock25();

    // LED's for debug maybe
    IOPort0.setPinsAsOutput(0x01|0x04);

    // P6 (IOPort5) has 6 bits of color output R0,R1, G0,G1, B0,B1
    // P4.1 (IOPort3) is HSync
    // P4.2 (IOPort3) is VSync
    IOPort5.setPinsAsOutput(0x3f);
    IOPort3.setPinsAsOutput(0x87);

    image = HackadayLogo.image;

    Timer.setInterval(800, 1);
    Timer.setListener(true);

    while(true);
  }

  public void timerInterrupt()
  {
    int a;

    // Vertical 640x480 (800x525) we need:
    // 2 lines front porch
    // 2 lines vertical sync (Vsync signal applied)
    // 25 lines back porch
    // 8 lines to border
    // 480 lines video (the RGB signals can be non-zero now)
    // 8 lines bottom border

    // Horizontal scanline in 640x480 (800x525) is
    // 8 pixels front porch
    // 96 pixels horizontal sync (Hsync signal applied)
    // 40 pixels back porch
    // 8 pixels left border
    // 640 pixels video (RBG signals can be non-zero here)
    // 8 pixels right border

    // NOTE: Internally it takes 17 cycles into the interrupt to make it
    // this far.  Interrupts take 6 cycles.  This should be a total of 23
    // cycles.

    // Turn on HSYNC for 96 cycles and vsync if rows 0 and 1
    IOPort3.setPinsValue(sync);
    for (a = 0; a < 6; a++);
    IOPort3.setPinHigh(HSYNC);
    IOPort3.setPinHigh(0);  // scope pin

    // Wait for back porch
    for (a = 0; a < 3; a++);

    // Draw a logo
    if (pos_y > 100 && pos_y < 400)
    {
      if (state == 0)
      {
        // Java Grinder optimizes for local variables better (at lesat
        // in this situation.  Also, counting downward can also be optimized
        // to use less instructions.
        int curr = ptr;
        a = 20;
        ptr += 20;
        do
        {
          IOPort5.setPinsValue(image[curr]);
          curr++;
          a--;
        } while(a != 0);
      }
        else
      {
        if (pos_y < 300)
        {
          // Inline assembly gets much higher resolution than straight Java.
          CPU.asm("  mov.w #40, r4\n" +
                  "  mov.w &image, r13\n" +
                  "  add.w &ptr, r13\n" +
                  "  add.w #40, &ptr\n" +
                  "draw_loop:\n" +
                  "  mov.b @r13+, &P6OUT\n" +
                  "  sub.w #1, r4\n" +
                  "  jnz draw_loop\n");
        }
      }
    }

    // Make sure RGB are off ...
    IOPort5.setPinsValue(0x00);

    pos_y++;

    if (pos_y >= 521)
    {
      pos_y = 0;
      sync = 0;
    }

    // Should Vsync be turned off
    if (pos_y == 2)
    {
      sync = (1<<VSYNC);
      ptr = 0;

      // Check if we should switch images.
      frame_count++;
      if (frame_count == 60 * 5)
      {
        frame_count = 0;
        state++;
        if (state == 4) { state = 0; }
        if (state == 0) { image = HackadayLogo.image; }
        else if (state == 1) { image = HackadayLogoHi.image; }
        else if (state == 2) { image = JavaLogo.image; }
        else if (state == 3) { image = TexasInstrumentsLogo.image; }
      }
    }
  }
}


